#!/usr/bin/env python
# -*- python -*-
#
#      Copyright (C) 2002-2004 Robert Muth <robert at muth dot org>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; version 2 of June 1991.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program in form of the file COPYING; 
#      if not, write to the 
#
#      Free Software Foundation, Inc. <http://www.fsf.org>
#      59 Temple Place, Suite 330, 
#      Boston, MA 02111-1307  USA
#
######################################################

#######################################################
#
# This module manages database files from cddb.com and freedb.org
#
#######################################################

import time
import string
import re 
import os
import sys

#######################################################
## 
#######################################################

patternDiscLength = re.compile(r'# Disc length:[ \t]+([0-9]+).*$')
patternTrackOffsets = re.compile(r'# Track frame offsets:[ \t\r]*$')
patternOffset = re.compile(r'#[ \t]+([0-9]+)[ \t\r]*$')
patternTocComment = re.compile(r'^[#\.](.*)$')
patternTocEmpty = re.compile(r'^[ \t\r\n]*$')
patternTocTrackTitle = re.compile(r'^TTITLE([0-9]+)\=(.*?)[\r]*$')
patternTocTrackExt = re.compile(r'^EXTT([0-9]+)\=(.*?)[\r]*$')
patternTocDiscTitle = re.compile(r'^DTITLE\=(.*?)[\r]*$')
patternTocDiscYear = re.compile(r'^DYEAR\=(.*?)[\r]*$')
patternTocDiscGenre = re.compile(r'^DGENRE\=(.*?)[\r]*$')
patternTocDiscId = re.compile(r'^DISCID\=(.*?)[\r]*$')
patternTocDiscExt = re.compile(r'^EXTD\=(.*?)[\r]*$')
patternTocPlayOrder = re.compile(r'^PLAYORDER\=(.*)$')

PatternFileNameWithLeadingDigits = re.compile(r'^([0-9]+)[^0-9].*$')
PatternFileNameWithLeadingDigits2 = re.compile(r'^track([0-9]+)[^0-9].*$')


class Toc:
    
    def __init__(self, filename):

        input = open(filename, "r")
        lines = input.readlines()
        input.close()

        self._discid = ""
        self._disctitle = ""
        self._discyear = ""
        self._discgenre = ""
        self._discext = ""
        self._artist = ""
        self._album = ""
        self._tracktitle= []
        self._trackext = []
        self._tracktimes = []
        self._disctime = None
        
        while lines:
            line = lines.pop(0)
#            line = unicode(lines.pop(0),'utf-8')
#            line = unicode(lines.pop(0),'latin_1')
            
            match = patternTrackOffsets.search(line)
            if match:
                while 1:
                    line = lines.pop(0)
                    match = patternOffset.search(line)
                    if match:
                        self._tracktimes.append( int(match.group(1)))
                    else:
                        break
                    # there is an important fall through here

            match = patternDiscLength.search(line)
            if match:
                self._disctime = int(match.group(1))
                self._tracktimes.append( self._disctime * 75 )
                for i in range(len(self._tracktimes)-1):
                     self._tracktimes[i] = (self._tracktimes[i+1]-self._tracktimes[i])/75
                self._tracktimes[-1] = self._tracktimes[-1]/75
                self._tracktitle = [""] * (len(self._tracktimes)-1)
                self._trackext = [""] * (len(self._tracktimes)-1)  
                continue


            if line[0] == '#' or line[0] == ' ' or line[0] == '\n' or line[0] == '\r' or line[0] == '\t' or line[0] == '.':
                continue

            match = patternTocPlayOrder.search(line)
            if match:
                continue
            match = patternTocDiscExt.search(line)
            if match:
                self._discext =   self._discext + match.group(1)
                continue
            match = patternTocDiscId.search(line)
            if match:
                self._discid =   self._discid + match.group(1)
                continue
            match = patternTocDiscGenre.search(line)
            if match != None:
                self._discgenre =   self._discgenre + match.group(1)
                continue
            match = patternTocDiscYear.search(line)
            if match != None:
                self._discyear =   self._discyear + match.group(1)
                continue
            match = patternTocDiscTitle.search(line)
            if match != None:
                self._disctitle =   self._disctitle + match.group(1)
                continue
            match = patternTocTrackTitle.search(line)
            if match != None:
                track =  int(match.group(1))
                self._tracktitle[track] = self._tracktitle[track] +  match.group(2)
                continue    
            match = patternTocTrackExt.search(line)
            if match != None:
                track =  int(match.group(1))
                self._trackext[track] = self._trackext[track] + match.group(2)
                continue

            assert(0 and ("BAD LINE:" + line) )

        index = self._disctitle.find("/")
        if index > 0:
            self._artist = self._disctitle[0:index]
            self._album = self._disctitle[(index+1):]
#        assert( self._album)
#        assert( self._artist)
        return
        
    def title_get(self):
        return self._disctitle

    def artist_get(self):
        return self._artist

    def album_get(self):
        return self._album

    def track_get(self,string):
        dummy, file = os.path.split(string)
        match = PatternFileNameWithLeadingDigits.search(file)
        if match == None:
            match = PatternFileNameWithLeadingDigits2.search(file)
        if match == None:
            return (None,0,0)
        index = int(match.group(1)) - 1
        if index < len(self._tracktitle) and index >= 0:
            return (self._tracktitle[index],self._trackext[index],self._tracktimes[index])
        else:
            return (None,0,0)

    
    def dump(self):
        print "DISCID:    " + self._discid
        print "DISCTITLE: " + self._disctitle
        print "DISKEXT:   " + self._discext
        print "TRACK TITLES"
        for k in range(len(self._tracktitle)):
            print "%2d:" % k,self._tracktitle[k]
        print "TRACK EXTS"
        for k in range(len(self._trackext)):        
            print "%2d:" % k,self._trackext[k]
        print "TRACK TIMES"
        for k in range(len(self._tracktimes)):        
            print "%2d:" % k,self._tracktimes[k]

        print "ARTIST:    " + self._artist
        print "ALBUM:     " + self._album
        return
#######################################################
#######################################################


class TocContainer:
    def __init__(self):
        self._container = {}

            
    def find_toc(self,dirname,use_cache=1):
        filename = os.path.join(dirname, "toc")

        if use_cache and self._container.has_key(filename):
            return self._container[filename]

        if not os.access(filename,os.R_OK):
            self._container[filename] = None
            return None
        else:
            toc = Toc(filename)
            self._container[filename] = toc
            return toc
             
#######################################################
#######################################################

if __name__ == '__main__':
   
      for i in sys.argv[1:]:
        print i
        if not os.access(i,os.R_OK):
            print "cannot open " + i
            continue;
        toc = Toc(i)
        toc.dump()

#######################################################
## eof
#######################################################







