#!/usr/bin/env python
# -*- python -*-
#
#      Copyright (C) 2002-2004 Robert Muth <robert@muth.org>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; version 2 of June 1991.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program in form of the file COPYING; 
#      if not, write to the 
#
#      Free Software Foundation, Inc. <http://www.fsf.org>
#      59 Temple Place, Suite 330, 
#      Boston, MA 02111-1307  USA
#
#######################################################
#######################################################

import re
import os
import sys
import popen2
import signal
import time

#######################################################
##
#######################################################

def SaveToFile(file, message):
    file = open(file,"wa")
    file.write( message )
    file.close()
    return

#######################################################
## 
#######################################################
logfile = None

def Message(prefix,message):
    global logfile
    if not logfile:
        logfile = open("musicus.log","wa")
    prefix  = prefix + " " + time.strftime("%Y/%m/%d %H:%M")
    logfile.write(prefix + ": " + message)

def Debug(message):
    Message("D",message)

def Info(message):
    Message("I",message)

def Warning(message):
    Message("W",message)

def Error(message):
    Message("E",message)
    sys.exit(-1)
    
#######################################################
## 
#######################################################
History = [("","")] * 20

def HistoryAppendCommand(c):
    global History
    History = History[1:]
    History.append((c,None))
    return

def HistoryAppendResult(r):
    global History
    History[-1] = (History[-1][0],r)
    return

def HistoryDump():
    global History
    Debug( "DEBUG HISTORY:" )
    Debug( str(History) )
    return

#######################################################
## FIXME horrible parser 
#######################################################

def tokenize(line):
#    print line
    token = []
    whitespace = ' \t\r\n'
    quotes = '\'"'
    state = 'n'
    curr = ""
    escape = 0
    
    for char in line:

        if escape == 1:
            curr = curr + char
            escape = 0
            continue
        
        if char == '\\':
            curr = curr + char
            escape = 1
            continue
        
        # whitespace delimit tokens unless we are inside a string
        if char in whitespace:
            if state in quotes:
                curr = curr + char
            elif curr != "":
                token.append(curr)
                curr = ""
            continue

        # quote terminates a string started by some quote 
        if char in quotes:
            if state == 'n':
                if curr != "":  #space between tokens please
                    Info("unparsable line " + line)
                    raise "whitespace missing"
                curr = curr + char
                state = char                
            elif state == char:
                curr = curr + char
                token.append(curr)
                curr = ""
                state = 'n'
                
            else:
                curr = curr + char

            continue

        curr = curr + char

    if state != 'n': raise "quote missing"
    if escape != 0: raise "escape char  missing"

    if curr != "":
        token.append(curr)
      
    return token

#######################################################
#
#######################################################

def process_result(result):
    lastline = result[-1]

    token = tokenize(lastline)
    if token[0] != "@":
        Info("malformed token\n") 
        raise "bad token"

    if token[1] == "ERROR":
        token[2] = 1
        return token[1:]

    elif token[1] == "OK":
        token[2] = 0
        return token[2:]
    else:
        Info("bad token")
        raise "bad token"

#######################################################
#######################################################
    
class Musicus:
    """
    This class handles the communication with the
    musicus program
    """

    def start_process(self):
#        self._process = popen2.Popen4("musicus -output libALSA.so",0)
        self._process = popen2.Popen4("musicus",0)
#        assert( self._process  )
        self._stdin = self._process.tochild
        self._stdout = self._process.fromchild

        def mytimeout(signum, frame):
            HistoryDump()
            Error("timeout\n")
            raise IOError

        signal.signal(signal.SIGALRM, mytimeout)
        return
    

    def __init__(self):
        Info("start musicus\n")
        self.start_process()
        
#        self._stdin, self._stdout = os.popen2("musicus","t",0)
#        print self._stdin
#        print self._stdout
        return

    def kill(self):
        Info("kill musicus")
        os.kill( self._process.pid,9)
#        self._stdin.close()
#        self._stdout.close()
        return

    def issue_commad(self,command):
        HistoryAppendCommand(command)
        try:
            signal.alarm(5)
            count = 0

#            Debug(command)
            self._stdin.write(command)
            result = []
            while 1:
                line = self._stdout.readline()
                result.append(line)
                if line != "" and line[0] == '@': break
                count = count + 1
                if count > 100:
                    Warning("command " + command ) 
                    Warning("too many lines in result\n" + str(result) + "\n")
                    signal.alarm(0)
                    HistoryAppendResult(result)
                    raise IOError #this is almost like a longjump
                
            signal.alarm(0)
            HistoryAppendResult(result)
            return result

        except IOError:
            HistoryDump()
            Warning("pipe broken, command: "+  command)
            Info("re start musicus\n")
            self.start_process()
            #raise "\nERROR\npipe to musicus player broken - was musicus installed?"
            return ['@ ERROR "musicus crashed"\n']

    def version(self):
        result = self.issue_commad("version\n")
        return process_result( result )

    def setvol(self,lvol,rvol):
        result = self.issue_commad("setvol %d %d\n" % (lvol,rvol))
        return process_result( result )

    def getvol(self):
        result = self.issue_commad("getvol\n")
        return process_result( result )
    
    def play(self,song):
        result = self.issue_commad("play \"%s\"\n" % song)
        return process_result( result )

    def stop(self):
        result = self.issue_commad("stop\n")
        return process_result( result )

    def pause(self):
        result = self.issue_commad("pause\n")
        return process_result( result )

    def unpause(self):
        result = self.issue_commad("unpause\n")
        return process_result( result )

    def getlength(self):
        result = self.issue_commad("getlength\n")
        return process_result( result )

    def getpos(self):
        result = self.issue_commad("getpos\n")
        return process_result( result )

    def setpos(self,pos):
        result = self.issue_commad("setpos %d\n" % pos)
        return process_result( result )

    def getinfo(self,file):
        result = self.issue_commad("getinfo2 \"%s\"\n" % file)
        return process_result( result )

    def settitleformat(self,format):
        result = self.issue_commad("settitleformat \"%s\"\n" % format)
        return process_result( result )    

###############################3#######################################################
## simple unit test- just provide some music file on commandline
#######################################################################################
            
if __name__ == '__main__':
    musicus = Musicus()

    print "#####################################"
    print "Global Player settings"
    print "#####################################"

    result = musicus.settitleformat("%p / %a - %t")
    print result
    
    result = musicus.getvol()
    print result

    result = musicus.setvol(99,99)
    print result
    
    result = musicus.version()
    print result
    

    print "#####################################"
    print "Getting Songinfo"
    print "#####################################"
    
    for i in sys.argv[1:]:
        print i
        print musicus.getinfo(i)

    print "#####################################"
    print "Playing Songs"
    print "#####################################"

    
    for i in sys.argv[1:]:
        print i

        result = musicus.play(i)
        print result

        result = musicus.getpos()
        print result

        time.sleep(1)
        
        result = musicus.getpos()
        print result

        result = musicus.setpos(2000)
        print result

        result = musicus.getpos()
        print result

        time.sleep(1)

        result = musicus.getpos()
        print result

        result = musicus.stop()
        print result


    print "#####################################"
    print "User Commands"
    print "#####################################"

        
    while 1:
        line = sys.stdin.readline()
        result = musicus.issue_commad(line)

        print result

#######################################################################################
## eof
#######################################################################################        
